# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import offline_user_data
from google.ads.googleads.v22.enums.types import conversion_adjustment_type
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "UploadConversionAdjustmentsRequest",
        "UploadConversionAdjustmentsResponse",
        "ConversionAdjustment",
        "RestatementValue",
        "GclidDateTimePair",
        "ConversionAdjustmentResult",
    },
)


class UploadConversionAdjustmentsRequest(proto.Message):
    r"""Request message for
    [ConversionAdjustmentUploadService.UploadConversionAdjustments][google.ads.googleads.v22.services.ConversionAdjustmentUploadService.UploadConversionAdjustments].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer performing
            the upload.
        conversion_adjustments (MutableSequence[google.ads.googleads.v22.services.types.ConversionAdjustment]):
            Required. The conversion adjustments that are
            being uploaded.
        partial_failure (bool):
            Required. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid. This should always be set to
            true.
            See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        job_id (int):
            Optional. Optional input to set job ID. Must be a
            non-negative number that is less than 2^31 if provided. If
            this field is not provided, the API will generate a job ID
            in the range [2^31, (2^63)-1]. The API will return the value
            for this request in the ``job_id`` field of the
            ``UploadConversionAdjustmentsResponse``.

            This field is a member of `oneof`_ ``_job_id``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    conversion_adjustments: MutableSequence["ConversionAdjustment"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="ConversionAdjustment",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    job_id: int = proto.Field(
        proto.INT32,
        number=5,
        optional=True,
    )


class UploadConversionAdjustmentsResponse(proto.Message):
    r"""Response message for
    [ConversionAdjustmentUploadService.UploadConversionAdjustments][google.ads.googleads.v22.services.ConversionAdjustmentUploadService.UploadConversionAdjustments].

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to conversion adjustment
            failures in the partial failure mode. Returned
            when all errors occur inside the adjustments. If
            any errors occur outside the adjustments (for
            example, auth errors), we return an RPC level
            error. See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        results (MutableSequence[google.ads.googleads.v22.services.types.ConversionAdjustmentResult]):
            Returned for successfully processed conversion adjustments.
            Proto will be empty for rows that received an error. Results
            are not returned when validate_only is true.
        job_id (int):
            Job ID for the upload batch.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["ConversionAdjustmentResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="ConversionAdjustmentResult",
        )
    )
    job_id: int = proto.Field(
        proto.INT64,
        number=3,
    )


class ConversionAdjustment(proto.Message):
    r"""A conversion adjustment.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gclid_date_time_pair (google.ads.googleads.v22.services.types.GclidDateTimePair):
            For adjustments, uniquely identifies a conversion that was
            reported without an order ID specified. If the
            adjustment_type is ENHANCEMENT, this value is optional but
            may be set in addition to the order_id.
        order_id (str):
            The order ID of the conversion to be
            adjusted. If the conversion was reported with an
            order ID specified, that order ID must be used
            as the identifier here. The order ID is required
            for enhancements.

            This field is a member of `oneof`_ ``_order_id``.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion adjustment.
            Note: Although this resource name consists of a
            customer id and a conversion action id,
            validation will ignore the customer id and use
            the conversion action id as the sole identifier
            of the conversion action.

            This field is a member of `oneof`_ ``_conversion_action``.
        adjustment_date_time (str):
            The date time at which the adjustment occurred. Must be
            after the conversion_date_time. The timezone must be
            specified. The format is "yyyy-mm-dd hh:mm:ss+\|-hh:mm", for
            example, "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_adjustment_date_time``.
        adjustment_type (google.ads.googleads.v22.enums.types.ConversionAdjustmentTypeEnum.ConversionAdjustmentType):
            The adjustment type.
        restatement_value (google.ads.googleads.v22.services.types.RestatementValue):
            Information needed to restate the
            conversion's value. Required for restatements.
            Should not be supplied for retractions. An error
            will be returned if provided for a retraction.
            NOTE: If you want to upload a second restatement
            with a different adjusted value, it must have a
            new, more recent, adjustment occurrence time.
            Otherwise, it will be treated as a duplicate of
            the previous restatement and ignored.
        user_identifiers (MutableSequence[google.ads.googleads.v22.common.types.UserIdentifier]):
            The user identifiers to enhance the original
            conversion. ConversionAdjustmentUploadService
            only accepts user identifiers in enhancements.
            The maximum number of user identifiers for each
            enhancement is 5.
        user_agent (str):
            The user agent to enhance the original conversion. This can
            be found in your user's HTTP request header when they
            convert on your web page. Example, "Mozilla/5.0 (iPhone; CPU
            iPhone OS 12_2 like Mac OS X)". User agent can only be
            specified in enhancements with user identifiers. This should
            match the user agent of the request that sent the original
            conversion so the conversion and its enhancement are either
            both attributed as same-device or both attributed as
            cross-device.

            This field is a member of `oneof`_ ``_user_agent``.
    """

    gclid_date_time_pair: "GclidDateTimePair" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="GclidDateTimePair",
    )
    order_id: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    adjustment_date_time: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    adjustment_type: (
        conversion_adjustment_type.ConversionAdjustmentTypeEnum.ConversionAdjustmentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=conversion_adjustment_type.ConversionAdjustmentTypeEnum.ConversionAdjustmentType,
    )
    restatement_value: "RestatementValue" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="RestatementValue",
    )
    user_identifiers: MutableSequence[offline_user_data.UserIdentifier] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=10,
            message=offline_user_data.UserIdentifier,
        )
    )
    user_agent: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )


class RestatementValue(proto.Message):
    r"""Contains information needed to restate a conversion's value.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        adjusted_value (float):
            The restated conversion value. This is the
            value of the conversion after restatement. For
            example, to change the value of a conversion
            from 100 to 70, an adjusted value of 70 should
            be reported. NOTE: If you want to upload a
            second restatement with a different adjusted
            value, it must have a new, more recent,
            adjustment occurrence time. Otherwise, it will
            be treated as a duplicate of the previous
            restatement and ignored.

            This field is a member of `oneof`_ ``_adjusted_value``.
        currency_code (str):
            The currency of the restated value. If not
            provided, then the default currency from the
            conversion action is used, and if that is not
            set then the account currency is used. This is
            the ISO 4217 3-character currency code for
            example, USD or EUR.

            This field is a member of `oneof`_ ``_currency_code``.
    """

    adjusted_value: float = proto.Field(
        proto.DOUBLE,
        number=3,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class GclidDateTimePair(proto.Message):
    r"""Uniquely identifies a conversion that was reported without an
    order ID specified.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gclid (str):
            Google click ID (gclid) associated with the
            original conversion for this adjustment.

            This field is a member of `oneof`_ ``_gclid``.
        conversion_date_time (str):
            The date time at which the original conversion for this
            adjustment occurred. The timezone must be specified. The
            format is "yyyy-mm-dd hh:mm:ss+\|-hh:mm", for example,
            "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_conversion_date_time``.
    """

    gclid: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    conversion_date_time: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class ConversionAdjustmentResult(proto.Message):
    r"""Information identifying a successfully processed
    ConversionAdjustment.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gclid_date_time_pair (google.ads.googleads.v22.services.types.GclidDateTimePair):
            The gclid and conversion date time of the
            conversion.
        order_id (str):
            The order ID of the conversion to be
            adjusted.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion adjustment.

            This field is a member of `oneof`_ ``_conversion_action``.
        adjustment_date_time (str):
            The date time at which the adjustment occurred. The format
            is "yyyy-mm-dd hh:mm:ss+\|-hh:mm", for example, "2019-01-01
            12:32:45-08:00".

            This field is a member of `oneof`_ ``_adjustment_date_time``.
        adjustment_type (google.ads.googleads.v22.enums.types.ConversionAdjustmentTypeEnum.ConversionAdjustmentType):
            The adjustment type.
    """

    gclid_date_time_pair: "GclidDateTimePair" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="GclidDateTimePair",
    )
    order_id: str = proto.Field(
        proto.STRING,
        number=10,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    adjustment_date_time: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    adjustment_type: (
        conversion_adjustment_type.ConversionAdjustmentTypeEnum.ConversionAdjustmentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=conversion_adjustment_type.ConversionAdjustmentTypeEnum.ConversionAdjustmentType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import conversion_action_category
from google.ads.googleads.v21.enums.types import (
    conversion_value_rule_set_status,
)
from google.ads.googleads.v21.enums.types import value_rule_set_attachment_type
from google.ads.googleads.v21.enums.types import value_rule_set_dimension


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionValueRuleSet",
    },
)


class ConversionValueRuleSet(proto.Message):
    r"""A conversion value rule set

    Attributes:
        resource_name (str):
            Immutable. The resource name of the conversion value rule
            set. Conversion value rule set resource names have the form:

            ``customers/{customer_id}/conversionValueRuleSets/{conversion_value_rule_set_id}``
        id (int):
            Output only. The ID of the conversion value
            rule set.
        conversion_value_rules (MutableSequence[str]):
            Resource names of rules within the rule set.
        dimensions (MutableSequence[google.ads.googleads.v21.enums.types.ValueRuleSetDimensionEnum.ValueRuleSetDimension]):
            Defines dimensions for Value Rule conditions.
            The condition types of value rules within this
            value rule set must be of these dimensions. The
            first entry in this list is the primary
            dimension of the included value rules. When
            using value rule primary dimension segmentation,
            conversion values will be segmented into the
            values adjusted by value rules and the original
            values, if some value rules apply.
        owner_customer (str):
            Output only. The resource name of the conversion value rule
            set's owner customer. When the value rule set is inherited
            from a manager customer, owner_customer will be the resource
            name of the manager whereas the customer in the
            resource_name will be of the requesting serving customer.
            \*\* Read-only \*\*
        attachment_type (google.ads.googleads.v21.enums.types.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType):
            Immutable. Defines the scope where the
            conversion value rule set is attached.
        campaign (str):
            The resource name of the campaign when the
            conversion value rule set is attached to a
            campaign.
        status (google.ads.googleads.v21.enums.types.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus):
            Output only. The status of the conversion value rule set.
            \*\* Read-only \*\*
        conversion_action_categories (MutableSequence[google.ads.googleads.v21.enums.types.ConversionActionCategoryEnum.ConversionActionCategory]):
            Immutable. The conversion action categories
            of the conversion value rule set.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    conversion_value_rules: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    dimensions: MutableSequence[
        value_rule_set_dimension.ValueRuleSetDimensionEnum.ValueRuleSetDimension
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=value_rule_set_dimension.ValueRuleSetDimensionEnum.ValueRuleSetDimension,
    )
    owner_customer: str = proto.Field(
        proto.STRING,
        number=5,
    )
    attachment_type: (
        value_rule_set_attachment_type.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=value_rule_set_attachment_type.ValueRuleSetAttachmentTypeEnum.ValueRuleSetAttachmentType,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=7,
    )
    status: (
        conversion_value_rule_set_status.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus
    ) = proto.Field(
        proto.ENUM,
        number=8,
        enum=conversion_value_rule_set_status.ConversionValueRuleSetStatusEnum.ConversionValueRuleSetStatus,
    )
    conversion_action_categories: MutableSequence[
        conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory
    ] = proto.RepeatedField(
        proto.ENUM,
        number=9,
        enum=conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomerUserAccessErrorEnum",
    },
)


class CustomerUserAccessErrorEnum(proto.Message):
    r"""Container for enum describing possible CustomerUserAccess
    errors.

    """

    class CustomerUserAccessError(proto.Enum):
        r"""Enum describing possible customer user access errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_USER_ID (2):
                There is no user associated with the user id
                specified.
            REMOVAL_DISALLOWED (3):
                Unable to remove the access between the user
                and customer.
            DISALLOWED_ACCESS_ROLE (4):
                Unable to add or update the access role as
                specified.
            LAST_ADMIN_USER_OF_SERVING_CUSTOMER (5):
                The user can't remove itself from an active
                serving customer if it's the last admin user and
                the customer doesn't have any owner manager
            LAST_ADMIN_USER_OF_MANAGER (6):
                Last admin user cannot be removed from a
                manager.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_USER_ID = 2
        REMOVAL_DISALLOWED = 3
        DISALLOWED_ACCESS_ROLE = 4
        LAST_ADMIN_USER_OF_SERVING_CUSTOMER = 5
        LAST_ADMIN_USER_OF_MANAGER = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AssetAutomationTypeEnum",
    },
)


class AssetAutomationTypeEnum(proto.Message):
    r"""Container for enum describing the type of asset automation."""

    class AssetAutomationType(proto.Enum):
        r"""The type of asset automation.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used as a return value only. Represents value
                unknown in this version.
            TEXT_ASSET_AUTOMATION (2):
                Text asset automation includes headlines and
                descriptions. By default, advertisers are
                opted-in for Performance Max and opted-out for
                Search.
            GENERATE_VERTICAL_YOUTUBE_VIDEOS (3):
                Converts horizontal video assets to vertical
                orientation using content-aware technology. By
                default, advertisers are opted in for
                DemandGenVideoResponsiveAd.
            GENERATE_SHORTER_YOUTUBE_VIDEOS (4):
                Shortens video assets to better capture user
                attention using content-aware technology. By
                default, advertisers are opted in for
                DemandGenVideoResponsiveAd.
            GENERATE_LANDING_PAGE_PREVIEW (5):
                Generates a preview of the landing page shown
                in the engagement panel.
                By using this feature, you confirm that you own
                all legal rights to the images on the landing
                page used by this account (or you have
                permission to share the images with Google). You
                hereby instruct Google to publish these images
                on your behalf for advertising or other
                commercial purposes.
            GENERATE_ENHANCED_YOUTUBE_VIDEOS (6):
                Generates video enhancements (vertical and
                shorter videos) for PMax campaigns. Opted in by
                default.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        TEXT_ASSET_AUTOMATION = 2
        GENERATE_VERTICAL_YOUTUBE_VIDEOS = 3
        GENERATE_SHORTER_YOUTUBE_VIDEOS = 4
        GENERATE_LANDING_PAGE_PREVIEW = 5
        GENERATE_ENHANCED_YOUTUBE_VIDEOS = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetOfflineEvaluationErrorReasonsEnum",
    },
)


class AssetOfflineEvaluationErrorReasonsEnum(proto.Message):
    r"""Provides the quality evaluation disapproval reason of an
    asset.

    """

    class AssetOfflineEvaluationErrorReasons(proto.Enum):
        r"""Enum describing the quality evaluation disapproval reason of
        an asset.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PRICE_ASSET_DESCRIPTION_REPEATS_ROW_HEADER (2):
                One or more descriptions repeats its
                corresponding row header.
            PRICE_ASSET_REPETITIVE_HEADERS (3):
                Price asset contains repetitive headers.
            PRICE_ASSET_HEADER_INCOMPATIBLE_WITH_PRICE_TYPE (4):
                Price item header is not relevant to the
                price type.
            PRICE_ASSET_DESCRIPTION_INCOMPATIBLE_WITH_ITEM_HEADER (5):
                Price item description is not relevant to the
                item header.
            PRICE_ASSET_DESCRIPTION_HAS_PRICE_QUALIFIER (6):
                Price asset has a price qualifier in a
                description.
            PRICE_ASSET_UNSUPPORTED_LANGUAGE (7):
                Unsupported language for price assets
            PRICE_ASSET_OTHER_ERROR (8):
                Human raters identified an issue with the
                price asset that isn't captured by other error
                reasons. The primary purpose of this value is to
                represent legacy FeedItem disapprovals that are
                no longer produced.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PRICE_ASSET_DESCRIPTION_REPEATS_ROW_HEADER = 2
        PRICE_ASSET_REPETITIVE_HEADERS = 3
        PRICE_ASSET_HEADER_INCOMPATIBLE_WITH_PRICE_TYPE = 4
        PRICE_ASSET_DESCRIPTION_INCOMPATIBLE_WITH_ITEM_HEADER = 5
        PRICE_ASSET_DESCRIPTION_HAS_PRICE_QUALIFIER = 6
        PRICE_ASSET_UNSUPPORTED_LANGUAGE = 7
        PRICE_ASSET_OTHER_ERROR = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "VideoThumbnailEnum",
    },
)


class VideoThumbnailEnum(proto.Message):
    r"""Defines the thumbnail to use for In-Display video ads. Note that
    DEFAULT_THUMBNAIL may have been uploaded by the user while
    thumbnails 1-3 are auto-generated from the video.

    """

    class VideoThumbnail(proto.Enum):
        r"""Enum listing the possible types of a video thumbnail.

        Values:
            UNSPECIFIED (0):
                The type has not been specified.
            UNKNOWN (1):
                The received value is not known in this
                version. This is a response-only value.
            DEFAULT_THUMBNAIL (2):
                The default thumbnail. Can be auto-generated
                or user-uploaded.
            THUMBNAIL_1 (3):
                Thumbnail 1, generated from the video.
            THUMBNAIL_2 (4):
                Thumbnail 2, generated from the video.
            THUMBNAIL_3 (5):
                Thumbnail 3, generated from the video.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DEFAULT_THUMBNAIL = 2
        THUMBNAIL_1 = 3
        THUMBNAIL_2 = 4
        THUMBNAIL_3 = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import media_type
from google.ads.googleads.v20.enums.types import mime_type as gage_mime_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "MediaFile",
        "MediaImage",
        "MediaBundle",
        "MediaAudio",
        "MediaVideo",
    },
)


class MediaFile(proto.Message):
    r"""A media file.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the media file. Media file
            resource names have the form:

            ``customers/{customer_id}/mediaFiles/{media_file_id}``
        id (int):
            Output only. The ID of the media file.

            This field is a member of `oneof`_ ``_id``.
        type_ (google.ads.googleads.v20.enums.types.MediaTypeEnum.MediaType):
            Immutable. Type of the media file.
        mime_type (google.ads.googleads.v20.enums.types.MimeTypeEnum.MimeType):
            Output only. The mime type of the media file.
        source_url (str):
            Immutable. The URL of where the original
            media file was downloaded from (or a file name).
            Only used for media of type AUDIO and IMAGE.

            This field is a member of `oneof`_ ``_source_url``.
        name (str):
            Immutable. The name of the media file. The
            name can be used by clients to help identify
            previously uploaded media.

            This field is a member of `oneof`_ ``_name``.
        file_size (int):
            Output only. The size of the media file in
            bytes.

            This field is a member of `oneof`_ ``_file_size``.
        image (google.ads.googleads.v20.resources.types.MediaImage):
            Immutable. Encapsulates an Image.

            This field is a member of `oneof`_ ``mediatype``.
        media_bundle (google.ads.googleads.v20.resources.types.MediaBundle):
            Immutable. A ZIP archive media the content of
            which contains HTML5 assets.

            This field is a member of `oneof`_ ``mediatype``.
        audio (google.ads.googleads.v20.resources.types.MediaAudio):
            Output only. Encapsulates an Audio.

            This field is a member of `oneof`_ ``mediatype``.
        video (google.ads.googleads.v20.resources.types.MediaVideo):
            Immutable. Encapsulates a Video.

            This field is a member of `oneof`_ ``mediatype``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )
    type_: media_type.MediaTypeEnum.MediaType = proto.Field(
        proto.ENUM,
        number=5,
        enum=media_type.MediaTypeEnum.MediaType,
    )
    mime_type: gage_mime_type.MimeTypeEnum.MimeType = proto.Field(
        proto.ENUM,
        number=6,
        enum=gage_mime_type.MimeTypeEnum.MimeType,
    )
    source_url: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    file_size: int = proto.Field(
        proto.INT64,
        number=15,
        optional=True,
    )
    image: "MediaImage" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="mediatype",
        message="MediaImage",
    )
    media_bundle: "MediaBundle" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="mediatype",
        message="MediaBundle",
    )
    audio: "MediaAudio" = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="mediatype",
        message="MediaAudio",
    )
    video: "MediaVideo" = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="mediatype",
        message="MediaVideo",
    )


class MediaImage(proto.Message):
    r"""Encapsulates an Image.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        data (bytes):
            Immutable. Raw image data.

            This field is a member of `oneof`_ ``_data``.
        full_size_image_url (str):
            Output only. The url to the full size version
            of the image.

            This field is a member of `oneof`_ ``_full_size_image_url``.
        preview_size_image_url (str):
            Output only. The url to the preview size
            version of the image.

            This field is a member of `oneof`_ ``_preview_size_image_url``.
    """

    data: bytes = proto.Field(
        proto.BYTES,
        number=4,
        optional=True,
    )
    full_size_image_url: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    preview_size_image_url: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )


class MediaBundle(proto.Message):
    r"""Represents a ZIP archive media the content of which contains
    HTML5 assets.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        data (bytes):
            Immutable. Raw zipped data.

            This field is a member of `oneof`_ ``_data``.
        url (str):
            Output only. The url to access the uploaded
            zipped data. For example,
            https://tpc.googlesyndication.com/simgad/123
            This field is read-only.

            This field is a member of `oneof`_ ``_url``.
    """

    data: bytes = proto.Field(
        proto.BYTES,
        number=3,
        optional=True,
    )
    url: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class MediaAudio(proto.Message):
    r"""Encapsulates an Audio.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        ad_duration_millis (int):
            Output only. The duration of the Audio in
            milliseconds.

            This field is a member of `oneof`_ ``_ad_duration_millis``.
    """

    ad_duration_millis: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class MediaVideo(proto.Message):
    r"""Encapsulates a Video.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        ad_duration_millis (int):
            Output only. The duration of the Video in
            milliseconds.

            This field is a member of `oneof`_ ``_ad_duration_millis``.
        youtube_video_id (str):
            Immutable. The YouTube video ID (as seen in
            YouTube URLs). Adding prefix
            "https://www.youtube.com/watch?v=" to this ID
            will get the YouTube streaming URL for this
            video.

            This field is a member of `oneof`_ ``_youtube_video_id``.
        advertising_id_code (str):
            Output only. The Advertising Digital
            Identification code for this video, as defined
            by the American Association of Advertising
            Agencies, used mainly for television
            commercials.

            This field is a member of `oneof`_ ``_advertising_id_code``.
        isci_code (str):
            Output only. The Industry Standard Commercial
            Identifier code for this video, used mainly for
            television commercials.

            This field is a member of `oneof`_ ``_isci_code``.
    """

    ad_duration_millis: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    youtube_video_id: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    advertising_id_code: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    isci_code: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import keyword_match_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "KeywordPlanCampaignKeyword",
    },
)


class KeywordPlanCampaignKeyword(proto.Message):
    r"""A Keyword Plan Campaign keyword.
    Only negative keywords are supported for Campaign Keyword.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Keyword Plan Campaign
            keyword. KeywordPlanCampaignKeyword resource names have the
            form:

            ``customers/{customer_id}/keywordPlanCampaignKeywords/{kp_campaign_keyword_id}``
        keyword_plan_campaign (str):
            The Keyword Plan campaign to which this
            negative keyword belongs.

            This field is a member of `oneof`_ ``_keyword_plan_campaign``.
        id (int):
            Output only. The ID of the Keyword Plan
            negative keyword.

            This field is a member of `oneof`_ ``_id``.
        text (str):
            The keyword text.

            This field is a member of `oneof`_ ``_text``.
        match_type (google.ads.googleads.v22.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
            The keyword match type.
        negative (bool):
            Immutable. If true, the keyword is negative.
            Must be set to true. Only negative campaign
            keywords are supported.

            This field is a member of `oneof`_ ``_negative``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    keyword_plan_campaign: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    id: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    text: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    match_type: keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
        )
    )
    negative: bool = proto.Field(
        proto.BOOL,
        number=11,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "UrlFieldErrorEnum",
    },
)


class UrlFieldErrorEnum(proto.Message):
    r"""Container for enum describing possible url field errors."""

    class UrlFieldError(proto.Enum):
        r"""Enum describing possible url field errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_TRACKING_URL_TEMPLATE (2):
                The tracking url template is invalid.
            INVALID_TAG_IN_TRACKING_URL_TEMPLATE (3):
                The tracking url template contains invalid
                tag.
            MISSING_TRACKING_URL_TEMPLATE_TAG (4):
                The tracking url template must contain at
                least one tag (for example, {lpurl}), This
                applies only to tracking url template associated
                with website ads or product ads.
            MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE (5):
                The tracking url template must start with a
                valid protocol (or lpurl tag).
            INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE (6):
                The tracking url template starts with an
                invalid protocol.
            MALFORMED_TRACKING_URL_TEMPLATE (7):
                The tracking url template contains illegal
                characters.
            MISSING_HOST_IN_TRACKING_URL_TEMPLATE (8):
                The tracking url template must contain a host
                name (or lpurl tag).
            INVALID_TLD_IN_TRACKING_URL_TEMPLATE (9):
                The tracking url template has an invalid or
                missing top level domain extension.
            REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG (10):
                The tracking url template contains nested
                occurrences of the same conditional tag (for
                example, {ifmobile:{ifmobile:x}}).
            INVALID_FINAL_URL (11):
                The final url is invalid.
            INVALID_TAG_IN_FINAL_URL (12):
                The final url contains invalid tag.
            REDUNDANT_NESTED_FINAL_URL_TAG (13):
                The final url contains nested occurrences of
                the same conditional tag (for example,
                {ifmobile:{ifmobile:x}}).
            MISSING_PROTOCOL_IN_FINAL_URL (14):
                The final url must start with a valid
                protocol.
            INVALID_PROTOCOL_IN_FINAL_URL (15):
                The final url starts with an invalid
                protocol.
            MALFORMED_FINAL_URL (16):
                The final url contains illegal characters.
            MISSING_HOST_IN_FINAL_URL (17):
                The final url must contain a host name.
            INVALID_TLD_IN_FINAL_URL (18):
                The tracking url template has an invalid or
                missing top level domain extension.
            INVALID_FINAL_MOBILE_URL (19):
                The final mobile url is invalid.
            INVALID_TAG_IN_FINAL_MOBILE_URL (20):
                The final mobile url contains invalid tag.
            REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG (21):
                The final mobile url contains nested
                occurrences of the same conditional tag (for
                example, {ifmobile:{ifmobile:x}}).
            MISSING_PROTOCOL_IN_FINAL_MOBILE_URL (22):
                The final mobile url must start with a valid
                protocol.
            INVALID_PROTOCOL_IN_FINAL_MOBILE_URL (23):
                The final mobile url starts with an invalid
                protocol.
            MALFORMED_FINAL_MOBILE_URL (24):
                The final mobile url contains illegal
                characters.
            MISSING_HOST_IN_FINAL_MOBILE_URL (25):
                The final mobile url must contain a host
                name.
            INVALID_TLD_IN_FINAL_MOBILE_URL (26):
                The tracking url template has an invalid or
                missing top level domain extension.
            INVALID_FINAL_APP_URL (27):
                The final app url is invalid.
            INVALID_TAG_IN_FINAL_APP_URL (28):
                The final app url contains invalid tag.
            REDUNDANT_NESTED_FINAL_APP_URL_TAG (29):
                The final app url contains nested occurrences
                of the same conditional tag (for example,
                {ifmobile:{ifmobile:x}}).
            MULTIPLE_APP_URLS_FOR_OSTYPE (30):
                More than one app url found for the same OS
                type.
            INVALID_OSTYPE (31):
                The OS type given for an app url is not
                valid.
            INVALID_PROTOCOL_FOR_APP_URL (32):
                The protocol given for an app url is not
                valid. (For example, "android-app://")
            INVALID_PACKAGE_ID_FOR_APP_URL (33):
                The package id (app id) given for an app url
                is not valid.
            URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT (34):
                The number of url custom parameters for an
                resource exceeds the maximum limit allowed.
            INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY (39):
                An invalid character appears in the parameter
                key.
            INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE (40):
                An invalid character appears in the parameter
                value.
            INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE (41):
                The url custom parameter value fails url tag
                validation.
            REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG (42):
                The custom parameter contains nested
                occurrences of the same conditional tag (for
                example, {ifmobile:{ifmobile:x}}).
            MISSING_PROTOCOL (43):
                The protocol (http:// or https://) is
                missing.
            INVALID_PROTOCOL (52):
                Unsupported protocol in URL. Only http and
                https are supported.
            INVALID_URL (44):
                The url is invalid.
            DESTINATION_URL_DEPRECATED (45):
                Destination Url is deprecated.
            INVALID_TAG_IN_URL (46):
                The url contains invalid tag.
            MISSING_URL_TAG (47):
                The url must contain at least one tag (for
                example, {lpurl}).
            DUPLICATE_URL_ID (48):
                Duplicate url id.
            INVALID_URL_ID (49):
                Invalid url id.
            FINAL_URL_SUFFIX_MALFORMED (50):
                The final url suffix cannot begin with '?' or
                '&' characters and must be a valid query string.
            INVALID_TAG_IN_FINAL_URL_SUFFIX (51):
                The final url suffix cannot contain {lpurl}
                related or {ignore} tags.
            INVALID_TOP_LEVEL_DOMAIN (53):
                The top level domain is invalid, for example,
                not a public top level domain listed in
                publicsuffix.org.
            MALFORMED_TOP_LEVEL_DOMAIN (54):
                Malformed top level domain in URL.
            MALFORMED_URL (55):
                Malformed URL.
            MISSING_HOST (56):
                No host found in URL.
            NULL_CUSTOM_PARAMETER_VALUE (57):
                Custom parameter value cannot be null.
            VALUE_TRACK_PARAMETER_NOT_SUPPORTED (58):
                Track parameter is not supported.
            UNSUPPORTED_APP_STORE (59):
                The app store connected to the url is not
                supported.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_TRACKING_URL_TEMPLATE = 2
        INVALID_TAG_IN_TRACKING_URL_TEMPLATE = 3
        MISSING_TRACKING_URL_TEMPLATE_TAG = 4
        MISSING_PROTOCOL_IN_TRACKING_URL_TEMPLATE = 5
        INVALID_PROTOCOL_IN_TRACKING_URL_TEMPLATE = 6
        MALFORMED_TRACKING_URL_TEMPLATE = 7
        MISSING_HOST_IN_TRACKING_URL_TEMPLATE = 8
        INVALID_TLD_IN_TRACKING_URL_TEMPLATE = 9
        REDUNDANT_NESTED_TRACKING_URL_TEMPLATE_TAG = 10
        INVALID_FINAL_URL = 11
        INVALID_TAG_IN_FINAL_URL = 12
        REDUNDANT_NESTED_FINAL_URL_TAG = 13
        MISSING_PROTOCOL_IN_FINAL_URL = 14
        INVALID_PROTOCOL_IN_FINAL_URL = 15
        MALFORMED_FINAL_URL = 16
        MISSING_HOST_IN_FINAL_URL = 17
        INVALID_TLD_IN_FINAL_URL = 18
        INVALID_FINAL_MOBILE_URL = 19
        INVALID_TAG_IN_FINAL_MOBILE_URL = 20
        REDUNDANT_NESTED_FINAL_MOBILE_URL_TAG = 21
        MISSING_PROTOCOL_IN_FINAL_MOBILE_URL = 22
        INVALID_PROTOCOL_IN_FINAL_MOBILE_URL = 23
        MALFORMED_FINAL_MOBILE_URL = 24
        MISSING_HOST_IN_FINAL_MOBILE_URL = 25
        INVALID_TLD_IN_FINAL_MOBILE_URL = 26
        INVALID_FINAL_APP_URL = 27
        INVALID_TAG_IN_FINAL_APP_URL = 28
        REDUNDANT_NESTED_FINAL_APP_URL_TAG = 29
        MULTIPLE_APP_URLS_FOR_OSTYPE = 30
        INVALID_OSTYPE = 31
        INVALID_PROTOCOL_FOR_APP_URL = 32
        INVALID_PACKAGE_ID_FOR_APP_URL = 33
        URL_CUSTOM_PARAMETERS_COUNT_EXCEEDS_LIMIT = 34
        INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_KEY = 39
        INVALID_CHARACTERS_IN_URL_CUSTOM_PARAMETER_VALUE = 40
        INVALID_TAG_IN_URL_CUSTOM_PARAMETER_VALUE = 41
        REDUNDANT_NESTED_URL_CUSTOM_PARAMETER_TAG = 42
        MISSING_PROTOCOL = 43
        INVALID_PROTOCOL = 52
        INVALID_URL = 44
        DESTINATION_URL_DEPRECATED = 45
        INVALID_TAG_IN_URL = 46
        MISSING_URL_TAG = 47
        DUPLICATE_URL_ID = 48
        INVALID_URL_ID = 49
        FINAL_URL_SUFFIX_MALFORMED = 50
        INVALID_TAG_IN_FINAL_URL_SUFFIX = 51
        INVALID_TOP_LEVEL_DOMAIN = 53
        MALFORMED_TOP_LEVEL_DOMAIN = 54
        MALFORMED_URL = 55
        MISSING_HOST = 56
        NULL_CUSTOM_PARAMETER_VALUE = 57
        VALUE_TRACK_PARAMETER_NOT_SUPPORTED = 58
        UNSUPPORTED_APP_STORE = 59


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "SearchTermInsightErrorEnum",
    },
)


class SearchTermInsightErrorEnum(proto.Message):
    r"""Container for enum describing possible search term insight
    errors.

    """

    class SearchTermInsightError(proto.Enum):
        r"""Enum describing possible search term insight errors.

        Values:
            UNSPECIFIED (0):
                Name unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FILTERING_NOT_ALLOWED_WITH_SEGMENTS (2):
                Search term insights cannot be filtered by
                metrics when segmenting.
            LIMIT_NOT_ALLOWED_WITH_SEGMENTS (3):
                Search term insights cannot have a LIMIT when
                segmenting.
            MISSING_FIELD_IN_SELECT_CLAUSE (4):
                A selected field requires another field to be
                selected with it.
            REQUIRES_FILTER_BY_SINGLE_RESOURCE (5):
                A selected field/resource requires filtering
                by a single resource.
            SORTING_NOT_ALLOWED_WITH_SEGMENTS (6):
                Search term insights cannot be sorted when
                segmenting.
            SUMMARY_ROW_NOT_ALLOWED_WITH_SEGMENTS (7):
                Search term insights cannot have a summary
                row when segmenting.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FILTERING_NOT_ALLOWED_WITH_SEGMENTS = 2
        LIMIT_NOT_ALLOWED_WITH_SEGMENTS = 3
        MISSING_FIELD_IN_SELECT_CLAUSE = 4
        REQUIRES_FILTER_BY_SINGLE_RESOURCE = 5
        SORTING_NOT_ALLOWED_WITH_SEGMENTS = 6
        SUMMARY_ROW_NOT_ALLOWED_WITH_SEGMENTS = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import product_availability
from google.ads.googleads.v19.enums.types import product_channel
from google.ads.googleads.v19.enums.types import product_channel_exclusivity
from google.ads.googleads.v19.enums.types import product_condition
from google.ads.googleads.v19.enums.types import product_issue_severity
from google.ads.googleads.v19.enums.types import product_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "ShoppingProduct",
    },
)


class ShoppingProduct(proto.Message):
    r"""A shopping product from Google Merchant Center that can be
    advertised by campaigns.

    The resource returns currently existing products from Google
    Merchant Center accounts linked with the customer. A campaign
    includes a product by specifying its merchant id (or, if available,
    the Multi Client Account id of the merchant) in the
    ``ShoppingSetting``, and can limit the inclusion to products having
    a specified feed label. Standard Shopping campaigns can also limit
    the inclusion through a ``campaign_criterion.listing_scope``.

    Queries to this resource specify a scope: Account:

    -  Filters on campaigns or ad groups are not specified.
    -  All products from the linked Google Merchant Center accounts are
       returned.
    -  Metrics and some fields (see the per-field documentation) are
       aggregated across all Shopping and Performance Max campaigns that
       include a product. Campaign:
    -  An equality filter on ``campaign`` is specified. Supported
       campaign types are Shopping, Performance Max, Demand Gen, Video.
    -  Only products that are included by the specified campaign are
       returned.
    -  Metrics and some fields (see the per-field documentation) are
       restricted to the specified campaign. Ad group:
    -  An equality filter on ``ad group`` and ``campaign`` is specified.
       Supported campaign types are Shopping, Demand Gen, Video.
    -  Only products that are included by the specified campaign are
       returned.
    -  Metrics and some fields (see the per-field documentation) are
       restricted to the specified ad group. Note that segmentation by
       date segments is not permitted and will return
       UNSUPPORTED_DATE_SEGMENTATION error. On the other hand, filtering
       on date segments is allowed.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the shopping product.
            Shopping product resource names have the form:

            ``customers/{customer_id}/shoppingProducts/{merchant_center_id}~{channel}~{language_code}~{feed_label}~{item_id}``
        merchant_center_id (int):
            Output only. The id of the merchant that owns
            the product.
        channel (google.ads.googleads.v19.enums.types.ProductChannelEnum.ProductChannel):
            Output only. The product channel describing
            the locality of the product.
        language_code (str):
            Output only. The language code as provided by
            the merchant, in BCP 47 format.
        feed_label (str):
            Output only. The product feed label as
            provided by the merchant.
        item_id (str):
            Output only. The item id of the product as
            provided by the merchant.
        multi_client_account_id (int):
            Output only. The id of the Multi Client
            Account of the merchant, if present.

            This field is a member of `oneof`_ ``_multi_client_account_id``.
        title (str):
            Output only. The title of the product as
            provided by the merchant.

            This field is a member of `oneof`_ ``_title``.
        brand (str):
            Output only. The brand of the product as
            provided by the merchant.

            This field is a member of `oneof`_ ``_brand``.
        price_micros (int):
            Output only. The price of the product in micros as provided
            by the merchant, in the currency specified in
            ``currency_code`` (e.g. $2.97 is reported as 2970000).

            This field is a member of `oneof`_ ``_price_micros``.
        currency_code (str):
            Output only. The currency code as provided by
            the merchant, in ISO 4217 format.

            This field is a member of `oneof`_ ``_currency_code``.
        channel_exclusivity (google.ads.googleads.v19.enums.types.ProductChannelExclusivityEnum.ProductChannelExclusivity):
            Output only. The channel exclusivity of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_channel_exclusivity``.
        condition (google.ads.googleads.v19.enums.types.ProductConditionEnum.ProductCondition):
            Output only. The condition of the product as
            provided by the merchant.

            This field is a member of `oneof`_ ``_condition``.
        availability (google.ads.googleads.v19.enums.types.ProductAvailabilityEnum.ProductAvailability):
            Output only. The availability of the product
            as provided by the merchant.

            This field is a member of `oneof`_ ``_availability``.
        target_countries (MutableSequence[str]):
            Output only. Upper-case two-letter ISO 3166-1
            code of the regions where the product is
            intended to be shown in ads.
        custom_attribute0 (str):
            Output only. The custom attribute 0 of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_custom_attribute0``.
        custom_attribute1 (str):
            Output only. The custom attribute 1 of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_custom_attribute1``.
        custom_attribute2 (str):
            Output only. The custom attribute 2 of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_custom_attribute2``.
        custom_attribute3 (str):
            Output only. The custom attribute 3 of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_custom_attribute3``.
        custom_attribute4 (str):
            Output only. The custom attribute 4 of the
            product as provided by the merchant.

            This field is a member of `oneof`_ ``_custom_attribute4``.
        category_level1 (str):
            Output only. The category level 1 of the
            product.

            This field is a member of `oneof`_ ``_category_level1``.
        category_level2 (str):
            Output only. The category level 2 of the
            product.

            This field is a member of `oneof`_ ``_category_level2``.
        category_level3 (str):
            Output only. The category level 3 of the
            product.

            This field is a member of `oneof`_ ``_category_level3``.
        category_level4 (str):
            Output only. The category level 4 of the
            product.

            This field is a member of `oneof`_ ``_category_level4``.
        category_level5 (str):
            Output only. The category level 5 of the
            product.

            This field is a member of `oneof`_ ``_category_level5``.
        product_type_level1 (str):
            Output only. The product type level 1 as
            provided by the merchant.

            This field is a member of `oneof`_ ``_product_type_level1``.
        product_type_level2 (str):
            Output only. The product type level 2 as
            provided by the merchant.

            This field is a member of `oneof`_ ``_product_type_level2``.
        product_type_level3 (str):
            Output only. The product type level 3 as
            provided by the merchant.

            This field is a member of `oneof`_ ``_product_type_level3``.
        product_type_level4 (str):
            Output only. The product type level 4 as
            provided by the merchant.

            This field is a member of `oneof`_ ``_product_type_level4``.
        product_type_level5 (str):
            Output only. The product type level 5 as
            provided by the merchant.

            This field is a member of `oneof`_ ``_product_type_level5``.
        effective_max_cpc_micros (int):
            Output only. The effective maximum
            cost-per-click (effective max. CPC) of the
            product. This field is available only if the
            query specifies the campaign or ad group scope,
            and if the campaign uses manual bidding. The
            value is the highest bid set for the product in
            product groups across all enabled ad groups. It
            represents the most you're willing to pay for a
            click on the product. This field can take up to
            24 hours to update.

            This field is a member of `oneof`_ ``_effective_max_cpc_micros``.
        status (google.ads.googleads.v19.enums.types.ProductStatusEnum.ProductStatus):
            Output only. The status that indicates
            whether the product can show in ads. The value
            of this field is restricted to the scope
            specified in the query, see the documentation of
            the resource.
            This field can take up to 24 hours to update.
        issues (MutableSequence[google.ads.googleads.v19.resources.types.ShoppingProduct.ProductIssue]):
            Output only. The list of issues affecting
            whether the product can show in ads. The value
            of this field is restricted to the scope
            specified in the query, see the documentation of
            the resource. This field can take up to 24 hours
            to update.
        campaign (str):
            Output only. A campaign that includes the product. This
            field is selectable only in the campaign scope, which
            requires an equality filter on ``campaign``.

            This field is a member of `oneof`_ ``_campaign``.
        ad_group (str):
            Output only. An ad group of a campaign that includes the
            product. This field is selectable only in the ad group
            scope, which requires an equality filter on ``campaign`` and
            ``ad_group``.

            This field is a member of `oneof`_ ``_ad_group``.
    """

    class ProductIssue(proto.Message):
        r"""An issue affecting whether a product can show in ads.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            error_code (str):
                Output only. The error code that identifies
                the issue.
            ads_severity (google.ads.googleads.v19.enums.types.ProductIssueSeverityEnum.ProductIssueSeverity):
                Output only. The severity of the issue in
                Google Ads.
            attribute_name (str):
                Output only. The name of the product's
                attribute, if any, that triggered the issue.

                This field is a member of `oneof`_ ``_attribute_name``.
            description (str):
                Output only. The short description of the
                issue in English.
            detail (str):
                Output only. The detailed description of the
                issue in English.
            documentation (str):
                Output only. The URL of the Help Center
                article for the issue.
            affected_regions (MutableSequence[str]):
                Output only. List of upper-case two-letter
                ISO 3166-1 codes of the regions affected by the
                issue. If empty, all regions are affected.
        """

        error_code: str = proto.Field(
            proto.STRING,
            number=1,
        )
        ads_severity: (
            product_issue_severity.ProductIssueSeverityEnum.ProductIssueSeverity
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=product_issue_severity.ProductIssueSeverityEnum.ProductIssueSeverity,
        )
        attribute_name: str = proto.Field(
            proto.STRING,
            number=3,
            optional=True,
        )
        description: str = proto.Field(
            proto.STRING,
            number=4,
        )
        detail: str = proto.Field(
            proto.STRING,
            number=5,
        )
        documentation: str = proto.Field(
            proto.STRING,
            number=6,
        )
        affected_regions: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=7,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    merchant_center_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    channel: product_channel.ProductChannelEnum.ProductChannel = proto.Field(
        proto.ENUM,
        number=3,
        enum=product_channel.ProductChannelEnum.ProductChannel,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=4,
    )
    feed_label: str = proto.Field(
        proto.STRING,
        number=5,
    )
    item_id: str = proto.Field(
        proto.STRING,
        number=6,
    )
    multi_client_account_id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    title: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    brand: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    price_micros: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    channel_exclusivity: (
        product_channel_exclusivity.ProductChannelExclusivityEnum.ProductChannelExclusivity
    ) = proto.Field(
        proto.ENUM,
        number=12,
        optional=True,
        enum=product_channel_exclusivity.ProductChannelExclusivityEnum.ProductChannelExclusivity,
    )
    condition: product_condition.ProductConditionEnum.ProductCondition = (
        proto.Field(
            proto.ENUM,
            number=13,
            optional=True,
            enum=product_condition.ProductConditionEnum.ProductCondition,
        )
    )
    availability: (
        product_availability.ProductAvailabilityEnum.ProductAvailability
    ) = proto.Field(
        proto.ENUM,
        number=14,
        optional=True,
        enum=product_availability.ProductAvailabilityEnum.ProductAvailability,
    )
    target_countries: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=15,
    )
    custom_attribute0: str = proto.Field(
        proto.STRING,
        number=16,
        optional=True,
    )
    custom_attribute1: str = proto.Field(
        proto.STRING,
        number=17,
        optional=True,
    )
    custom_attribute2: str = proto.Field(
        proto.STRING,
        number=18,
        optional=True,
    )
    custom_attribute3: str = proto.Field(
        proto.STRING,
        number=19,
        optional=True,
    )
    custom_attribute4: str = proto.Field(
        proto.STRING,
        number=20,
        optional=True,
    )
    category_level1: str = proto.Field(
        proto.STRING,
        number=21,
        optional=True,
    )
    category_level2: str = proto.Field(
        proto.STRING,
        number=22,
        optional=True,
    )
    category_level3: str = proto.Field(
        proto.STRING,
        number=23,
        optional=True,
    )
    category_level4: str = proto.Field(
        proto.STRING,
        number=24,
        optional=True,
    )
    category_level5: str = proto.Field(
        proto.STRING,
        number=25,
        optional=True,
    )
    product_type_level1: str = proto.Field(
        proto.STRING,
        number=26,
        optional=True,
    )
    product_type_level2: str = proto.Field(
        proto.STRING,
        number=27,
        optional=True,
    )
    product_type_level3: str = proto.Field(
        proto.STRING,
        number=28,
        optional=True,
    )
    product_type_level4: str = proto.Field(
        proto.STRING,
        number=29,
        optional=True,
    )
    product_type_level5: str = proto.Field(
        proto.STRING,
        number=30,
        optional=True,
    )
    effective_max_cpc_micros: int = proto.Field(
        proto.INT64,
        number=31,
        optional=True,
    )
    status: product_status.ProductStatusEnum.ProductStatus = proto.Field(
        proto.ENUM,
        number=32,
        enum=product_status.ProductStatusEnum.ProductStatus,
    )
    issues: MutableSequence[ProductIssue] = proto.RepeatedField(
        proto.MESSAGE,
        number=33,
        message=ProductIssue,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=34,
        optional=True,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=35,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "MobileAppVendorEnum",
    },
)


class MobileAppVendorEnum(proto.Message):
    r"""Container for enum describing different types of mobile app
    vendors.

    """

    class MobileAppVendor(proto.Enum):
        r"""The type of mobile app vendor

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            APPLE_APP_STORE (2):
                Mobile app vendor for Apple app store.
            GOOGLE_APP_STORE (3):
                Mobile app vendor for Google app store.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        APPLE_APP_STORE = 2
        GOOGLE_APP_STORE = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "RegenerateShareableLinkIdRequest",
        "RegenerateShareableLinkIdResponse",
    },
)


class RegenerateShareableLinkIdRequest(proto.Message):
    r"""Request message for
    [ThirdPartyAppAnalyticsLinkService.RegenerateShareableLinkId][google.ads.googleads.v19.services.ThirdPartyAppAnalyticsLinkService.RegenerateShareableLinkId].

    Attributes:
        resource_name (str):
            Resource name of the third party app
            analytics link.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RegenerateShareableLinkIdResponse(proto.Message):
    r"""Response message for
    [ThirdPartyAppAnalyticsLinkService.RegenerateShareableLinkId][google.ads.googleads.v19.services.ThirdPartyAppAnalyticsLinkService.RegenerateShareableLinkId].

    """


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "InteractionEventTypeEnum",
    },
)


class InteractionEventTypeEnum(proto.Message):
    r"""Container for enum describing types of payable and free
    interactions.

    """

    class InteractionEventType(proto.Enum):
        r"""Enum describing possible types of payable and free
        interactions.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CLICK (2):
                Click to site. In most cases, this
                interaction navigates to an external location,
                usually the advertiser's landing page. This is
                also the default InteractionEventType for click
                events.
            ENGAGEMENT (3):
                The user's expressed intent to engage with
                the ad in-place.
            VIDEO_VIEW (4):
                User viewed a video ad.
            NONE (5):
                The default InteractionEventType for ad
                conversion events. This is used when an ad
                conversion row does NOT indicate that the free
                interactions (for example, the ad conversions)
                should be 'promoted' and reported as part of the
                core metrics. These are simply other (ad)
                conversions.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CLICK = 2
        ENGAGEMENT = 3
        VIDEO_VIEW = 4
        NONE = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import criteria
from google.ads.googleads.v21.common.types import extensions
from google.ads.googleads.v21.enums.types import (
    ad_group_type as gage_ad_group_type,
)
from google.ads.googleads.v21.enums.types import (
    advertising_channel_type as gage_advertising_channel_type,
)
from google.ads.googleads.v21.enums.types import (
    bidding_strategy_type as gage_bidding_strategy_type,
)
from google.ads.googleads.v21.enums.types import conversion_tracking_status_enum
from google.ads.googleads.v21.enums.types import keyword_match_type
from google.ads.googleads.v21.enums.types import recommendation_type
from google.ads.googleads.v21.enums.types import (
    target_impression_share_location,
)
from google.ads.googleads.v21.resources.types import ad as gagr_ad
from google.ads.googleads.v21.resources.types import asset
from google.ads.googleads.v21.resources.types import recommendation
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "ApplyRecommendationRequest",
        "ApplyRecommendationOperation",
        "ApplyRecommendationResponse",
        "ApplyRecommendationResult",
        "DismissRecommendationRequest",
        "DismissRecommendationResponse",
        "GenerateRecommendationsRequest",
        "GenerateRecommendationsResponse",
    },
)


class ApplyRecommendationRequest(proto.Message):
    r"""Request message for
    [RecommendationService.ApplyRecommendation][google.ads.googleads.v21.services.RecommendationService.ApplyRecommendation].

    Attributes:
        customer_id (str):
            Required. The ID of the customer with the
            recommendation.
        operations (MutableSequence[google.ads.googleads.v21.services.types.ApplyRecommendationOperation]):
            Required. The list of operations to apply recommendations.
            If partial_failure=false all recommendations should be of
            the same type There is a limit of 100 operations per
            request.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, operations will be carried out
            as a transaction if and only if they are all
            valid. Default is false.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["ApplyRecommendationOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="ApplyRecommendationOperation",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class ApplyRecommendationOperation(proto.Message):
    r"""Information about the operation to apply a recommendation and
    any parameters to customize it.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            The resource name of the recommendation to
            apply.
        campaign_budget (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.CampaignBudgetParameters):
            Optional parameters to use when applying a
            campaign budget recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        text_ad (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.TextAdParameters):
            Optional parameters to use when applying a
            text ad recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        keyword (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.KeywordParameters):
            Optional parameters to use when applying
            keyword recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        target_cpa_opt_in (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.TargetCpaOptInParameters):
            Optional parameters to use when applying
            target CPA opt-in recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        target_roas_opt_in (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.TargetRoasOptInParameters):
            Optional parameters to use when applying
            target ROAS opt-in recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        callout_extension (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.CalloutExtensionParameters):
            Parameters to use when applying callout
            extension recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        call_extension (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.CallExtensionParameters):
            Parameters to use when applying call
            extension recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        sitelink_extension (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.SitelinkExtensionParameters):
            Parameters to use when applying sitelink
            recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        move_unused_budget (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.MoveUnusedBudgetParameters):
            Parameters to use when applying move unused
            budget recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ResponsiveSearchAdParameters):
            Parameters to use when applying a responsive
            search ad recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        use_broad_match_keyword (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.UseBroadMatchKeywordParameters):
            Parameters to use when applying a use broad
            match keyword recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad_asset (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters):
            Parameters to use when applying a responsive
            search ad asset recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad_improve_ad_strength (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters):
            Parameters to use when applying a responsive
            search ad improve ad strength recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        raise_target_cpa_bid_too_low (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.RaiseTargetCpaBidTooLowParameters):
            Parameters to use when applying a raise
            target CPA bid too low recommendation. The apply
            is asynchronous and can take minutes depending
            on the number of ad groups there is in the
            related campaign.

            This field is a member of `oneof`_ ``apply_parameters``.
        forecasting_set_target_roas (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ForecastingSetTargetRoasParameters):
            Parameters to use when applying a forecasting
            set target ROAS recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        callout_asset (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.CalloutAssetParameters):
            Parameters to use when applying callout asset
            recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        call_asset (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.CallAssetParameters):
            Parameters to use when applying call asset
            recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        sitelink_asset (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.SitelinkAssetParameters):
            Parameters to use when applying sitelink
            asset recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        raise_target_cpa (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.RaiseTargetCpaParameters):
            Parameters to use when applying raise Target
            CPA recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        lower_target_roas (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.LowerTargetRoasParameters):
            Parameters to use when applying lower Target
            ROAS recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        forecasting_set_target_cpa (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ForecastingSetTargetCpaParameters):
            Parameters to use when applying forecasting
            set target CPA recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        set_target_cpa (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ForecastingSetTargetCpaParameters):
            Parameters to use when applying set target
            CPA recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        set_target_roas (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.ForecastingSetTargetRoasParameters):
            Parameters to use when applying set target
            ROAS recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        lead_form_asset (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.LeadFormAssetParameters):
            Parameters to use when applying lead form
            asset recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
    """

    class CampaignBudgetParameters(proto.Message):
        r"""Parameters to use when applying a campaign budget
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            new_budget_amount_micros (int):
                New budget amount to set for target budget
                resource. This is a required field.

                This field is a member of `oneof`_ ``_new_budget_amount_micros``.
        """

        new_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class ForecastingSetTargetRoasParameters(proto.Message):
        r"""Parameters to use when applying a forecasting set target roas
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_roas (float):
                New target ROAS (revenue per unit of spend)
                to set for a campaign resource.
                The value is between 0.01 and 1000.0, inclusive.

                This field is a member of `oneof`_ ``_target_roas``.
            campaign_budget_amount_micros (int):
                New campaign budget amount to set for a
                campaign resource.

                This field is a member of `oneof`_ ``_campaign_budget_amount_micros``.
        """

        target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
            optional=True,
        )
        campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class TextAdParameters(proto.Message):
        r"""Parameters to use when applying a text ad recommendation.

        Attributes:
            ad (google.ads.googleads.v21.resources.types.Ad):
                New ad to add to recommended ad group. All
                necessary fields need to be set in this message.
                This is a required field.
        """

        ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )

    class KeywordParameters(proto.Message):
        r"""Parameters to use when applying keyword recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            ad_group (str):
                The ad group resource to add keyword to. This
                is a required field.

                This field is a member of `oneof`_ ``_ad_group``.
            match_type (google.ads.googleads.v21.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
                The match type of the keyword. This is a
                required field.
            cpc_bid_micros (int):
                Optional, CPC bid to set for the keyword. If
                not set, keyword will use bid based on bidding
                strategy used by target ad group.

                This field is a member of `oneof`_ ``_cpc_bid_micros``.
        """

        ad_group: str = proto.Field(
            proto.STRING,
            number=4,
            optional=True,
        )
        match_type: keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType = (
            proto.Field(
                proto.ENUM,
                number=2,
                enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
            )
        )
        cpc_bid_micros: int = proto.Field(
            proto.INT64,
            number=5,
            optional=True,
        )

    class TargetCpaOptInParameters(proto.Message):
        r"""Parameters to use when applying Target CPA recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_cpa_micros (int):
                Average CPA to use for Target CPA bidding
                strategy. This is a required field.

                This field is a member of `oneof`_ ``_target_cpa_micros``.
            new_campaign_budget_amount_micros (int):
                Optional, budget amount to set for the
                campaign.

                This field is a member of `oneof`_ ``_new_campaign_budget_amount_micros``.
        """

        target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )
        new_campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=4,
            optional=True,
        )

    class TargetRoasOptInParameters(proto.Message):
        r"""Parameters to use when applying a Target ROAS opt-in
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_roas (float):
                Average ROAS (revenue per unit of spend) to use for Target
                ROAS bidding strategy. The value is between 0.01 and 1000.0,
                inclusive. This is a required field, unless
                new_campaign_budget_amount_micros is set.

                This field is a member of `oneof`_ ``_target_roas``.
            new_campaign_budget_amount_micros (int):
                Optional, budget amount to set for the
                campaign.

                This field is a member of `oneof`_ ``_new_campaign_budget_amount_micros``.
        """

        target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
            optional=True,
        )
        new_campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class CalloutExtensionParameters(proto.Message):
        r"""Parameters to use when applying callout extension
        recommendation.

        Attributes:
            callout_extensions (MutableSequence[google.ads.googleads.v21.common.types.CalloutFeedItem]):
                Callout extensions to be added. This is a
                required field.
        """

        callout_extensions: MutableSequence[extensions.CalloutFeedItem] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=extensions.CalloutFeedItem,
            )
        )

    class CallExtensionParameters(proto.Message):
        r"""Parameters to use when applying call extension
        recommendation.

        Attributes:
            call_extensions (MutableSequence[google.ads.googleads.v21.common.types.CallFeedItem]):
                Call extensions to be added. This is a
                required field.
        """

        call_extensions: MutableSequence[extensions.CallFeedItem] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=extensions.CallFeedItem,
            )
        )

    class SitelinkExtensionParameters(proto.Message):
        r"""Parameters to use when applying sitelink recommendation.

        Attributes:
            sitelink_extensions (MutableSequence[google.ads.googleads.v21.common.types.SitelinkFeedItem]):
                Sitelinks to be added. This is a required
                field.
        """

        sitelink_extensions: MutableSequence[extensions.SitelinkFeedItem] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=extensions.SitelinkFeedItem,
            )
        )

    class CalloutAssetParameters(proto.Message):
        r"""Parameters to use when applying callout asset
        recommendations.

        Attributes:
            ad_asset_apply_parameters (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.AdAssetApplyParameters):
                Required. Callout assets to be added. This is
                a required field.
        """

        ad_asset_apply_parameters: (
            "ApplyRecommendationOperation.AdAssetApplyParameters"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ApplyRecommendationOperation.AdAssetApplyParameters",
        )

    class CallAssetParameters(proto.Message):
        r"""Parameters to use when applying call asset recommendations.

        Attributes:
            ad_asset_apply_parameters (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.AdAssetApplyParameters):
                Required. Call assets to be added. This is a
                required field.
        """

        ad_asset_apply_parameters: (
            "ApplyRecommendationOperation.AdAssetApplyParameters"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ApplyRecommendationOperation.AdAssetApplyParameters",
        )

    class SitelinkAssetParameters(proto.Message):
        r"""Parameters to use when applying sitelink asset
        recommendations.

        Attributes:
            ad_asset_apply_parameters (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.AdAssetApplyParameters):
                Required. Sitelink assets to be added. This
                is a required field.
        """

        ad_asset_apply_parameters: (
            "ApplyRecommendationOperation.AdAssetApplyParameters"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ApplyRecommendationOperation.AdAssetApplyParameters",
        )

    class RaiseTargetCpaParameters(proto.Message):
        r"""Parameters to use when applying raise Target CPA
        recommendations.

        Attributes:
            target_cpa_multiplier (float):
                Required. Target to set CPA multiplier to.
                This is a required field.
        """

        target_cpa_multiplier: float = proto.Field(
            proto.DOUBLE,
            number=1,
        )

    class LowerTargetRoasParameters(proto.Message):
        r"""Parameters to use when applying lower Target ROAS
        recommendations.

        Attributes:
            target_roas_multiplier (float):
                Required. Target to set ROAS multiplier to.
                This is a required field.
        """

        target_roas_multiplier: float = proto.Field(
            proto.DOUBLE,
            number=1,
        )

    class AdAssetApplyParameters(proto.Message):
        r"""Common parameters used when applying ad asset
        recommendations.

        Attributes:
            new_assets (MutableSequence[google.ads.googleads.v21.resources.types.Asset]):
                The assets to create and attach to a scope. This may be
                combined with existing_assets in the same call.
            existing_assets (MutableSequence[str]):
                The resource names of existing assets to attach to a scope.
                This may be combined with new_assets in the same call.
            scope (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.AdAssetApplyParameters.ApplyScope):
                Required. The scope at which to apply the
                assets. Assets at the campaign scope level will
                be applied to the campaign associated with the
                recommendation. Assets at the customer scope
                will apply to the entire account. Assets at the
                campaign scope will override any attached at the
                customer scope.
        """

        class ApplyScope(proto.Enum):
            r"""Scope to apply the assets to.

            Values:
                UNSPECIFIED (0):
                    The apply scope has not been specified.
                UNKNOWN (1):
                    Unknown.
                CUSTOMER (2):
                    Apply at the customer scope.
                CAMPAIGN (3):
                    Apply at the campaign scope.
            """

            UNSPECIFIED = 0
            UNKNOWN = 1
            CUSTOMER = 2
            CAMPAIGN = 3

        new_assets: MutableSequence[asset.Asset] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=asset.Asset,
        )
        existing_assets: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )
        scope: (
            "ApplyRecommendationOperation.AdAssetApplyParameters.ApplyScope"
        ) = proto.Field(
            proto.ENUM,
            number=3,
            enum="ApplyRecommendationOperation.AdAssetApplyParameters.ApplyScope",
        )

    class MoveUnusedBudgetParameters(proto.Message):
        r"""Parameters to use when applying move unused budget
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            budget_micros_to_move (int):
                Budget amount to move from excess budget to
                constrained budget. This is a required field.

                This field is a member of `oneof`_ ``_budget_micros_to_move``.
        """

        budget_micros_to_move: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class ResponsiveSearchAdAssetParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad asset
        recommendation.

        Attributes:
            updated_ad (google.ads.googleads.v21.resources.types.Ad):
                Updated ad. The current ad's content will be
                replaced.
        """

        updated_ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdImproveAdStrengthParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad
        improve ad strength recommendation.

        Attributes:
            updated_ad (google.ads.googleads.v21.resources.types.Ad):
                Updated ad. The current ad's content will be
                replaced.
        """

        updated_ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad
        recommendation.

        Attributes:
            ad (google.ads.googleads.v21.resources.types.Ad):
                Required. New ad to add to recommended ad
                group.
        """

        ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )

    class RaiseTargetCpaBidTooLowParameters(proto.Message):
        r"""Parameters to use when applying a raise target CPA bid too
        low recommendation. The apply is asynchronous and can take
        minutes depending on the number of ad groups there is in the
        related campaign..

        Attributes:
            target_multiplier (float):
                Required. A number greater than 1.0
                indicating the factor by which to increase the
                target CPA. This is a required field.
        """

        target_multiplier: float = proto.Field(
            proto.DOUBLE,
            number=1,
        )

    class UseBroadMatchKeywordParameters(proto.Message):
        r"""Parameters to use when applying a use broad match keyword
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            new_budget_amount_micros (int):
                New budget amount to set for target budget
                resource.

                This field is a member of `oneof`_ ``_new_budget_amount_micros``.
        """

        new_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )

    class ForecastingSetTargetCpaParameters(proto.Message):
        r"""Parameters to use when applying a set target CPA
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_cpa_micros (int):
                Average CPA to use for Target CPA bidding
                strategy.

                This field is a member of `oneof`_ ``_target_cpa_micros``.
            campaign_budget_amount_micros (int):
                New campaign budget amount to set for a
                campaign resource.

                This field is a member of `oneof`_ ``_campaign_budget_amount_micros``.
        """

        target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )
        campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class LeadFormAssetParameters(proto.Message):
        r"""Parameters to use when applying a lead form asset
        recommendation.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            ad_asset_apply_parameters (google.ads.googleads.v21.services.types.ApplyRecommendationOperation.AdAssetApplyParameters):
                Required. Lead form assets to be added. This
                is a required field.
            set_submit_lead_form_asset_campaign_goal (bool):
                If true, the "Submit Lead Form" goal will be
                set on the target campaign. As a result, ads
                will be shown as lead form creative ads. If
                false, the "Submit Lead Form" goal will not be
                set on the campaign and ads will contain lead
                form assets.

                This field is a member of `oneof`_ ``_set_submit_lead_form_asset_campaign_goal``.
        """

        ad_asset_apply_parameters: (
            "ApplyRecommendationOperation.AdAssetApplyParameters"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="ApplyRecommendationOperation.AdAssetApplyParameters",
        )
        set_submit_lead_form_asset_campaign_goal: bool = proto.Field(
            proto.BOOL,
            number=2,
            optional=True,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign_budget: CampaignBudgetParameters = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="apply_parameters",
        message=CampaignBudgetParameters,
    )
    text_ad: TextAdParameters = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="apply_parameters",
        message=TextAdParameters,
    )
    keyword: KeywordParameters = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="apply_parameters",
        message=KeywordParameters,
    )
    target_cpa_opt_in: TargetCpaOptInParameters = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="apply_parameters",
        message=TargetCpaOptInParameters,
    )
    target_roas_opt_in: TargetRoasOptInParameters = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="apply_parameters",
        message=TargetRoasOptInParameters,
    )
    callout_extension: CalloutExtensionParameters = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="apply_parameters",
        message=CalloutExtensionParameters,
    )
    call_extension: CallExtensionParameters = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="apply_parameters",
        message=CallExtensionParameters,
    )
    sitelink_extension: SitelinkExtensionParameters = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="apply_parameters",
        message=SitelinkExtensionParameters,
    )
    move_unused_budget: MoveUnusedBudgetParameters = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="apply_parameters",
        message=MoveUnusedBudgetParameters,
    )
    responsive_search_ad: ResponsiveSearchAdParameters = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="apply_parameters",
        message=ResponsiveSearchAdParameters,
    )
    use_broad_match_keyword: UseBroadMatchKeywordParameters = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="apply_parameters",
        message=UseBroadMatchKeywordParameters,
    )
    responsive_search_ad_asset: ResponsiveSearchAdAssetParameters = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="apply_parameters",
        message=ResponsiveSearchAdAssetParameters,
    )
    responsive_search_ad_improve_ad_strength: (
        ResponsiveSearchAdImproveAdStrengthParameters
    ) = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="apply_parameters",
        message=ResponsiveSearchAdImproveAdStrengthParameters,
    )
    raise_target_cpa_bid_too_low: RaiseTargetCpaBidTooLowParameters = (
        proto.Field(
            proto.MESSAGE,
            number=15,
            oneof="apply_parameters",
            message=RaiseTargetCpaBidTooLowParameters,
        )
    )
    forecasting_set_target_roas: ForecastingSetTargetRoasParameters = (
        proto.Field(
            proto.MESSAGE,
            number=16,
            oneof="apply_parameters",
            message=ForecastingSetTargetRoasParameters,
        )
    )
    callout_asset: CalloutAssetParameters = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="apply_parameters",
        message=CalloutAssetParameters,
    )
    call_asset: CallAssetParameters = proto.Field(
        proto.MESSAGE,
        number=18,
        oneof="apply_parameters",
        message=CallAssetParameters,
    )
    sitelink_asset: SitelinkAssetParameters = proto.Field(
        proto.MESSAGE,
        number=19,
        oneof="apply_parameters",
        message=SitelinkAssetParameters,
    )
    raise_target_cpa: RaiseTargetCpaParameters = proto.Field(
        proto.MESSAGE,
        number=20,
        oneof="apply_parameters",
        message=RaiseTargetCpaParameters,
    )
    lower_target_roas: LowerTargetRoasParameters = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="apply_parameters",
        message=LowerTargetRoasParameters,
    )
    forecasting_set_target_cpa: ForecastingSetTargetCpaParameters = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="apply_parameters",
        message=ForecastingSetTargetCpaParameters,
    )
    set_target_cpa: ForecastingSetTargetCpaParameters = proto.Field(
        proto.MESSAGE,
        number=23,
        oneof="apply_parameters",
        message=ForecastingSetTargetCpaParameters,
    )
    set_target_roas: ForecastingSetTargetRoasParameters = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="apply_parameters",
        message=ForecastingSetTargetRoasParameters,
    )
    lead_form_asset: LeadFormAssetParameters = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="apply_parameters",
        message=LeadFormAssetParameters,
    )


class ApplyRecommendationResponse(proto.Message):
    r"""Response message for
    [RecommendationService.ApplyRecommendation][google.ads.googleads.v21.services.RecommendationService.ApplyRecommendation].

    Attributes:
        results (MutableSequence[google.ads.googleads.v21.services.types.ApplyRecommendationResult]):
            Results of operations to apply
            recommendations.
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors) we return
            the RPC level error.
    """

    results: MutableSequence["ApplyRecommendationResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ApplyRecommendationResult",
    )
    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )


class ApplyRecommendationResult(proto.Message):
    r"""The result of applying a recommendation.

    Attributes:
        resource_name (str):
            Returned for successful applies.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DismissRecommendationRequest(proto.Message):
    r"""Request message for
    [RecommendationService.DismissRecommendation][google.ads.googleads.v21.services.RecommendationService.DismissRecommendation].

    Attributes:
        customer_id (str):
            Required. The ID of the customer with the
            recommendation.
        operations (MutableSequence[google.ads.googleads.v21.services.types.DismissRecommendationRequest.DismissRecommendationOperation]):
            Required. The list of operations to dismiss recommendations.
            If partial_failure=false all recommendations should be of
            the same type There is a limit of 100 operations per
            request.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, operations will be carried in
            a single transaction if and only if they are all
            valid. Default is false.
    """

    class DismissRecommendationOperation(proto.Message):
        r"""Operation to dismiss a single recommendation identified by
        resource_name.

        Attributes:
            resource_name (str):
                The resource name of the recommendation to
                dismiss.
        """

        resource_name: str = proto.Field(
            proto.STRING,
            number=1,
        )

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence[DismissRecommendationOperation] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message=DismissRecommendationOperation,
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class DismissRecommendationResponse(proto.Message):
    r"""Response message for
    [RecommendationService.DismissRecommendation][google.ads.googleads.v21.services.RecommendationService.DismissRecommendation].

    Attributes:
        results (MutableSequence[google.ads.googleads.v21.services.types.DismissRecommendationResponse.DismissRecommendationResult]):
            Results of operations to dismiss
            recommendations.
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors) we return
            the RPC level error.
    """

    class DismissRecommendationResult(proto.Message):
        r"""The result of dismissing a recommendation.

        Attributes:
            resource_name (str):
                Returned for successful dismissals.
        """

        resource_name: str = proto.Field(
            proto.STRING,
            number=1,
        )

    results: MutableSequence[DismissRecommendationResult] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=DismissRecommendationResult,
    )
    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )


class GenerateRecommendationsRequest(proto.Message):
    r"""Request message for
    [RecommendationService.GenerateRecommendations][google.ads.googleads.v21.services.RecommendationService.GenerateRecommendations].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer generating
            recommendations.
        recommendation_types (MutableSequence[google.ads.googleads.v21.enums.types.RecommendationTypeEnum.RecommendationType]):
            Required. List of eligible recommendation_types to generate.
            If the uploaded criteria isn't sufficient to make a
            recommendation, or the campaign is already in the
            recommended state, no recommendation will be returned for
            that type. Generally, a recommendation is returned if all
            required fields for that recommendation_type are uploaded,
            but there are cases where this is still not sufficient.

            The following recommendation_types are supported for
            recommendation generation: CAMPAIGN_BUDGET, KEYWORD,
            MAXIMIZE_CLICKS_OPT_IN, MAXIMIZE_CONVERSIONS_OPT_IN,
            MAXIMIZE_CONVERSION_VALUE_OPT_IN, SET_TARGET_CPA,
            SET_TARGET_ROAS, SITELINK_ASSET, TARGET_CPA_OPT_IN,
            TARGET_ROAS_OPT_IN
        advertising_channel_type (google.ads.googleads.v21.enums.types.AdvertisingChannelTypeEnum.AdvertisingChannelType):
            Required. Advertising channel type of the campaign. The
            following advertising_channel_types are supported for
            recommendation generation: PERFORMANCE_MAX and SEARCH
        campaign_sitelink_count (int):
            Optional. Number of sitelinks on the campaign. This field is
            necessary for the following recommendation_types:
            SITELINK_ASSET

            This field is a member of `oneof`_ ``_campaign_sitelink_count``.
        conversion_tracking_status (google.ads.googleads.v21.enums.types.ConversionTrackingStatusEnum.ConversionTrackingStatus):
            Optional. Current conversion tracking status. This field is
            necessary for the following recommendation_types:
            MAXIMIZE_CLICKS_OPT_IN, MAXIMIZE_CONVERSIONS_OPT_IN,
            MAXIMIZE_CONVERSION_VALUE_OPT_IN, SET_TARGET_CPA,
            SET_TARGET_ROAS, TARGET_CPA_OPT_IN, TARGET_ROAS_OPT_IN

            This field is a member of `oneof`_ ``_conversion_tracking_status``.
        bidding_info (google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.BiddingInfo):
            Optional. Current bidding information of the campaign. This
            field is necessary for the following recommendation_types:
            CAMPAIGN_BUDGET, MAXIMIZE_CLICKS_OPT_IN,
            MAXIMIZE_CONVERSIONS_OPT_IN,
            MAXIMIZE_CONVERSION_VALUE_OPT_IN, SET_TARGET_CPA,
            SET_TARGET_ROAS, TARGET_CPA_OPT_IN, TARGET_ROAS_OPT_IN

            This field is a member of `oneof`_ ``_bidding_info``.
        ad_group_info (MutableSequence[google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.AdGroupInfo]):
            Optional. Current AdGroup Information. Supports information
            from a single AdGroup. This field is optional for the
            following recommendation_types: KEYWORD This field is
            required for the following recommendation_types:
            CAMPAIGN_BUDGET if AdvertisingChannelType is SEARCH
        seed_info (google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.SeedInfo):
            Optional. Seed information for Keywords. This field is
            necessary for the following recommendation_types: KEYWORD

            This field is a member of `oneof`_ ``_seed_info``.
        budget_info (google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.BudgetInfo):
            Optional. Current budget information. This field is optional
            for the following recommendation_types: CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_budget_info``.
        campaign_image_asset_count (int):
            Optional. Current campaign image asset count. This field is
            optional for the following recommendation_types:
            CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_campaign_image_asset_count``.
        campaign_call_asset_count (int):
            Optional. Current campaign call asset count. This field is
            optional for the following recommendation_types:
            CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_campaign_call_asset_count``.
        country_codes (MutableSequence[str]):
            Optional. Current campaign country codes. This field is
            required for the following recommendation_types:
            CAMPAIGN_BUDGET if AdvertisingChannelType is SEARCH
        language_codes (MutableSequence[str]):
            Optional. Current campaign language codes. This field is
            required for the following recommendation_types:
            CAMPAIGN_BUDGET if AdvertisingChannelType is SEARCH
        positive_locations_ids (MutableSequence[int]):
            Optional. Current campaign positive location ids. One of
            this field OR negative_location_ids is required for the
            following recommendation_types: CAMPAIGN_BUDGET if
            AdvertisingChannelType is SEARCH
        negative_locations_ids (MutableSequence[int]):
            Optional. Current campaign negative location ids. One of
            this field OR positive_location_ids is required for the
            following recommendation_types: CAMPAIGN_BUDGET if
            AdvertisingChannelType is SEARCH
        asset_group_info (MutableSequence[google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.AssetGroupInfo]):
            Optional. Current AssetGroup Information. This field is
            required for the following recommendation_types:
            CAMPAIGN_BUDGET
        target_partner_search_network (bool):
            Optional. If true, the campaign is opted into serving ads on
            the Google Partner Network. This field is optional for the
            following recommendation_types: CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_target_partner_search_network``.
        target_content_network (bool):
            Optional. If true, the campaign is opted into serving ads on
            specified placements in the Google Display Network. This
            field is optional for the following recommendation_types:
            CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_target_content_network``.
        merchant_center_account_id (int):
            Optional. Merchant Center account ID. This field should only
            be set when advertising_channel_type is PERFORMANCE_MAX.
            Setting this field causes RecommendationService to generate
            recommendations for Performance Max for retail instead of
            standard Performance Max. This field is optional for the
            following recommendation_types: CAMPAIGN_BUDGET

            This field is a member of `oneof`_ ``_merchant_center_account_id``.
    """

    class BiddingInfo(proto.Message):
        r"""Current bidding information of the campaign. Provides a
        wrapper for bidding-related signals that inform recommendations.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            bidding_strategy_type (google.ads.googleads.v21.enums.types.BiddingStrategyTypeEnum.BiddingStrategyType):
                Current bidding strategy. This field is necessary for the
                following recommendation_types: CAMPAIGN_BUDGET,
                MAXIMIZE_CLICKS_OPT_IN, MAXIMIZE_CONVERSIONS_OPT_IN,
                MAXIMIZE_CONVERSION_VALUE_OPT_IN, SET_TARGET_CPA,
                SET_TARGET_ROAS, TARGET_CPA_OPT_IN, TARGET_ROAS_OPT_IN

                This field is a member of `oneof`_ ``_bidding_strategy_type``.
            target_cpa_micros (int):
                Current target_cpa in micros. This can be populated for
                campaigns with a bidding strategy type of TARGET_CPA or
                MAXIMIZE_CONVERSIONS.

                This field is a member of `oneof`_ ``bidding_strategy_target_info``.
            target_roas (float):
                Current target_roas. This can be populated for campaigns
                with a bidding strategy type of TARGET_ROAS or
                MAXIMIZE_CONVERSION_VALUE.

                This field is a member of `oneof`_ ``bidding_strategy_target_info``.
            target_impression_share_info (google.ads.googleads.v21.services.types.GenerateRecommendationsRequest.TargetImpressionShareInfo):
                Optional. Current Target Impression Share information of the
                campaign. This field is necessary for the following
                recommendation_types: CAMPAIGN_BUDGET

                This field is a member of `oneof`_ ``bidding_strategy_target_info``.
        """

        bidding_strategy_type: (
            gage_bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            optional=True,
            enum=gage_bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType,
        )
        target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=2,
            oneof="bidding_strategy_target_info",
        )
        target_roas: float = proto.Field(
            proto.DOUBLE,
            number=3,
            oneof="bidding_strategy_target_info",
        )
        target_impression_share_info: (
            "GenerateRecommendationsRequest.TargetImpressionShareInfo"
        ) = proto.Field(
            proto.MESSAGE,
            number=4,
            oneof="bidding_strategy_target_info",
            message="GenerateRecommendationsRequest.TargetImpressionShareInfo",
        )

    class AdGroupInfo(proto.Message):
        r"""Current AdGroup Information of the campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            ad_group_type (google.ads.googleads.v21.enums.types.AdGroupTypeEnum.AdGroupType):
                Optional. AdGroup Type of the AdGroup. This field is
                necessary for the following recommendation_types if
                ad_group_info is set: KEYWORD

                This field is a member of `oneof`_ ``_ad_group_type``.
            keywords (MutableSequence[google.ads.googleads.v21.common.types.KeywordInfo]):
                Optional. Current keywords. This field is optional for the
                following recommendation_types if ad_group_info is set:
                KEYWORD This field is required for the following
                recommendation_types: CAMPAIGN_BUDGET if
                AdvertisingChannelType is SEARCH
        """

        ad_group_type: gage_ad_group_type.AdGroupTypeEnum.AdGroupType = (
            proto.Field(
                proto.ENUM,
                number=1,
                optional=True,
                enum=gage_ad_group_type.AdGroupTypeEnum.AdGroupType,
            )
        )
        keywords: MutableSequence[criteria.KeywordInfo] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=criteria.KeywordInfo,
        )

    class SeedInfo(proto.Message):
        r"""A keyword seed and a specific url to generate keywords from.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            url_seed (str):
                A specific url to generate ideas from, for
                example: www.example.com/cars.

                This field is a member of `oneof`_ ``_url_seed``.
            keyword_seeds (MutableSequence[str]):
                Optional. Keywords or phrases to generate
                ideas from, for example: cars or "car dealership
                near me".
        """

        url_seed: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )
        keyword_seeds: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )

    class BudgetInfo(proto.Message):
        r"""Current budget information of the campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            current_budget (int):
                Required. Current budget amount in micros. This field is
                necessary for the following recommendation_types if
                budget_info is set: CAMPAIGN_BUDGET

                This field is a member of `oneof`_ ``_current_budget``.
        """

        current_budget: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )

    class AssetGroupInfo(proto.Message):
        r"""Current AssetGroup information of the campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            final_url (str):
                Required. Current url of the asset group. This field is
                necessary for the following recommendation_types if
                asset_group_info is set: CAMPAIGN_BUDGET

                This field is a member of `oneof`_ ``_final_url``.
            headline (MutableSequence[str]):
                Optional. Current headlines of the asset group. This field
                is optional for the following recommendation_types if
                asset_group_info is set: CAMPAIGN_BUDGET
            description (MutableSequence[str]):
                Optional. Current descriptions of the asset group. This
                field is optional for the following recommendation_types if
                asset_group_info is set: CAMPAIGN_BUDGET
        """

        final_url: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        headline: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )
        description: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )

    class TargetImpressionShareInfo(proto.Message):
        r"""Current Target Impression Share information of the campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            location (google.ads.googleads.v21.enums.types.TargetImpressionShareLocationEnum.TargetImpressionShareLocation):
                Required. The targeted location on the search results page.
                This is required for campaigns where the
                AdvertisingChannelType is SEARCH and the bidding strategy
                type is TARGET_IMPRESSION_SHARE.

                This field is a member of `oneof`_ ``_location``.
            target_impression_share_micros (int):
                Required. The chosen fraction of targeted impression share
                in micros. For example, 1% equals 10,000. It must be a value
                between 1 and 1,000,000. This is required for campaigns with
                an AdvertisingChannelType of SEARCH and a bidding strategy
                type of TARGET_IMPRESSION_SHARE.

                This field is a member of `oneof`_ ``_target_impression_share_micros``.
            max_cpc_bid_ceiling (int):
                Optional. Ceiling of max CPC bids in micros set by automated
                bidders. This is optional for campaigns with an
                AdvertisingChannelType of SEARCH and a bidding strategy type
                of TARGET_IMPRESSION_SHARE.

                This field is a member of `oneof`_ ``_max_cpc_bid_ceiling``.
        """

        location: (
            target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation
        ) = proto.Field(
            proto.ENUM,
            number=1,
            optional=True,
            enum=target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation,
        )
        target_impression_share_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )
        max_cpc_bid_ceiling: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    recommendation_types: MutableSequence[
        recommendation_type.RecommendationTypeEnum.RecommendationType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=recommendation_type.RecommendationTypeEnum.RecommendationType,
    )
    advertising_channel_type: (
        gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType,
    )
    campaign_sitelink_count: int = proto.Field(
        proto.INT32,
        number=4,
        optional=True,
    )
    conversion_tracking_status: (
        conversion_tracking_status_enum.ConversionTrackingStatusEnum.ConversionTrackingStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        optional=True,
        enum=conversion_tracking_status_enum.ConversionTrackingStatusEnum.ConversionTrackingStatus,
    )
    bidding_info: BiddingInfo = proto.Field(
        proto.MESSAGE,
        number=6,
        optional=True,
        message=BiddingInfo,
    )
    ad_group_info: MutableSequence[AdGroupInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=AdGroupInfo,
    )
    seed_info: SeedInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        optional=True,
        message=SeedInfo,
    )
    budget_info: BudgetInfo = proto.Field(
        proto.MESSAGE,
        number=9,
        optional=True,
        message=BudgetInfo,
    )
    campaign_image_asset_count: int = proto.Field(
        proto.INT32,
        number=10,
        optional=True,
    )
    campaign_call_asset_count: int = proto.Field(
        proto.INT32,
        number=11,
        optional=True,
    )
    country_codes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=13,
    )
    language_codes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=14,
    )
    positive_locations_ids: MutableSequence[int] = proto.RepeatedField(
        proto.INT64,
        number=15,
    )
    negative_locations_ids: MutableSequence[int] = proto.RepeatedField(
        proto.INT64,
        number=16,
    )
    asset_group_info: MutableSequence[AssetGroupInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=17,
        message=AssetGroupInfo,
    )
    target_partner_search_network: bool = proto.Field(
        proto.BOOL,
        number=18,
        optional=True,
    )
    target_content_network: bool = proto.Field(
        proto.BOOL,
        number=19,
        optional=True,
    )
    merchant_center_account_id: int = proto.Field(
        proto.INT64,
        number=20,
        optional=True,
    )


class GenerateRecommendationsResponse(proto.Message):
    r"""Response message for
    [RecommendationService.GenerateRecommendations][google.ads.googleads.v21.services.RecommendationService.GenerateRecommendations].

    Attributes:
        recommendations (MutableSequence[google.ads.googleads.v21.resources.types.Recommendation]):
            List of generated recommendations from the passed in set of
            requested recommendation_types. If there isn't sufficient
            data to generate a recommendation for the requested
            recommendation_types, the result set won't contain a
            recommendation for that type.
    """

    recommendations: MutableSequence[recommendation.Recommendation] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=recommendation.Recommendation,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "VanityPharmaTextEnum",
    },
)


class VanityPharmaTextEnum(proto.Message):
    r"""The text that will be displayed in display URL of the text ad
    when website description is the selected display mode for vanity
    pharma URLs.

    """

    class VanityPharmaText(proto.Enum):
        r"""Enum describing possible text.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PRESCRIPTION_TREATMENT_WEBSITE_EN (2):
                Prescription treatment website with website
                content in English.
            PRESCRIPTION_TREATMENT_WEBSITE_ES (3):
                Prescription treatment website with website
                content in Spanish (Sitio de tratamientos con
                receta).
            PRESCRIPTION_DEVICE_WEBSITE_EN (4):
                Prescription device website with website
                content in English.
            PRESCRIPTION_DEVICE_WEBSITE_ES (5):
                Prescription device website with website
                content in Spanish (Sitio de dispositivos con
                receta).
            MEDICAL_DEVICE_WEBSITE_EN (6):
                Medical device website with website content
                in English.
            MEDICAL_DEVICE_WEBSITE_ES (7):
                Medical device website with website content
                in Spanish (Sitio de dispositivos médicos).
            PREVENTATIVE_TREATMENT_WEBSITE_EN (8):
                Preventative treatment website with website
                content in English.
            PREVENTATIVE_TREATMENT_WEBSITE_ES (9):
                Preventative treatment website with website
                content in Spanish (Sitio de tratamientos
                preventivos).
            PRESCRIPTION_CONTRACEPTION_WEBSITE_EN (10):
                Prescription contraception website with
                website content in English.
            PRESCRIPTION_CONTRACEPTION_WEBSITE_ES (11):
                Prescription contraception website with
                website content in Spanish (Sitio de
                anticonceptivos con receta).
            PRESCRIPTION_VACCINE_WEBSITE_EN (12):
                Prescription vaccine website with website
                content in English.
            PRESCRIPTION_VACCINE_WEBSITE_ES (13):
                Prescription vaccine website with website
                content in Spanish (Sitio de vacunas con
                receta).
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PRESCRIPTION_TREATMENT_WEBSITE_EN = 2
        PRESCRIPTION_TREATMENT_WEBSITE_ES = 3
        PRESCRIPTION_DEVICE_WEBSITE_EN = 4
        PRESCRIPTION_DEVICE_WEBSITE_ES = 5
        MEDICAL_DEVICE_WEBSITE_EN = 6
        MEDICAL_DEVICE_WEBSITE_ES = 7
        PREVENTATIVE_TREATMENT_WEBSITE_EN = 8
        PREVENTATIVE_TREATMENT_WEBSITE_ES = 9
        PRESCRIPTION_CONTRACEPTION_WEBSITE_EN = 10
        PRESCRIPTION_CONTRACEPTION_WEBSITE_ES = 11
        PRESCRIPTION_VACCINE_WEBSITE_EN = 12
        PRESCRIPTION_VACCINE_WEBSITE_ES = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import frequency_cap_event_type
from google.ads.googleads.v21.enums.types import frequency_cap_level
from google.ads.googleads.v21.enums.types import frequency_cap_time_unit


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "FrequencyCapEntry",
        "FrequencyCapKey",
    },
)


class FrequencyCapEntry(proto.Message):
    r"""A rule specifying the maximum number of times an ad (or some
    set of ads) can be shown to a user over a particular time
    period.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        key (google.ads.googleads.v21.common.types.FrequencyCapKey):
            The key of a particular frequency cap. There
            can be no more than one frequency cap with the
            same key.
        cap (int):
            Maximum number of events allowed during the
            time range by this cap.

            This field is a member of `oneof`_ ``_cap``.
    """

    key: "FrequencyCapKey" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="FrequencyCapKey",
    )
    cap: int = proto.Field(
        proto.INT32,
        number=3,
        optional=True,
    )


class FrequencyCapKey(proto.Message):
    r"""A group of fields used as keys for a frequency cap.
    There can be no more than one frequency cap with the same key.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        level (google.ads.googleads.v21.enums.types.FrequencyCapLevelEnum.FrequencyCapLevel):
            The level on which the cap is to be applied
            (for example, ad group ad, ad group). The cap is
            applied to all the entities of this level.
        event_type (google.ads.googleads.v21.enums.types.FrequencyCapEventTypeEnum.FrequencyCapEventType):
            The type of event that the cap applies to
            (for example, impression).
        time_unit (google.ads.googleads.v21.enums.types.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit):
            Unit of time the cap is defined at (for
            example, day, week).
        time_length (int):
            Number of time units the cap lasts.

            This field is a member of `oneof`_ ``_time_length``.
    """

    level: frequency_cap_level.FrequencyCapLevelEnum.FrequencyCapLevel = (
        proto.Field(
            proto.ENUM,
            number=1,
            enum=frequency_cap_level.FrequencyCapLevelEnum.FrequencyCapLevel,
        )
    )
    event_type: (
        frequency_cap_event_type.FrequencyCapEventTypeEnum.FrequencyCapEventType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=frequency_cap_event_type.FrequencyCapEventTypeEnum.FrequencyCapEventType,
    )
    time_unit: (
        frequency_cap_time_unit.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=frequency_cap_time_unit.FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit,
    )
    time_length: int = proto.Field(
        proto.INT32,
        number=5,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import experiment_metric
from google.ads.googleads.v22.enums.types import experiment_metric_direction


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "MetricGoal",
    },
)


class MetricGoal(proto.Message):
    r"""A metric goal for an experiment.

    Attributes:
        metric (google.ads.googleads.v22.enums.types.ExperimentMetricEnum.ExperimentMetric):
            The metric of the goal. For example, clicks,
            impressions, cost, conversions, etc.
        direction (google.ads.googleads.v22.enums.types.ExperimentMetricDirectionEnum.ExperimentMetricDirection):
            The metric direction of the goal. For
            example, increase, decrease, no change.
    """

    metric: experiment_metric.ExperimentMetricEnum.ExperimentMetric = (
        proto.Field(
            proto.ENUM,
            number=1,
            enum=experiment_metric.ExperimentMetricEnum.ExperimentMetric,
        )
    )
    direction: (
        experiment_metric_direction.ExperimentMetricDirectionEnum.ExperimentMetricDirection
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=experiment_metric_direction.ExperimentMetricDirectionEnum.ExperimentMetricDirection,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupAdErrorEnum",
    },
)


class AdGroupAdErrorEnum(proto.Message):
    r"""Container for enum describing possible ad group ad errors."""

    class AdGroupAdError(proto.Enum):
        r"""Enum describing possible ad group ad errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            AD_GROUP_AD_LABEL_DOES_NOT_EXIST (2):
                No link found between the adgroup ad and the
                label.
            AD_GROUP_AD_LABEL_ALREADY_EXISTS (3):
                The label has already been attached to the
                adgroup ad.
            AD_NOT_UNDER_ADGROUP (4):
                The specified ad was not found in the adgroup
            CANNOT_OPERATE_ON_REMOVED_ADGROUPAD (5):
                Removed ads may not be modified
            CANNOT_CREATE_DEPRECATED_ADS (6):
                An ad of this type is deprecated and cannot
                be created. Only deletions are permitted.
            CANNOT_CREATE_TEXT_ADS (7):
                Text ads are deprecated and cannot be
                created. Use expanded text ads instead.
            EMPTY_FIELD (8):
                A required field was not specified or is an
                empty string.
            RESOURCE_REFERENCED_IN_MULTIPLE_OPS (9):
                An ad may only be modified once per call
            AD_TYPE_CANNOT_BE_PAUSED (10):
                AdGroupAds with the given ad type cannot be
                paused.
            AD_TYPE_CANNOT_BE_REMOVED (11):
                AdGroupAds with the given ad type cannot be
                removed.
            CANNOT_UPDATE_DEPRECATED_ADS (12):
                An ad of this type is deprecated and cannot
                be updated. Only removals are permitted.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_GROUP_AD_LABEL_DOES_NOT_EXIST = 2
        AD_GROUP_AD_LABEL_ALREADY_EXISTS = 3
        AD_NOT_UNDER_ADGROUP = 4
        CANNOT_OPERATE_ON_REMOVED_ADGROUPAD = 5
        CANNOT_CREATE_DEPRECATED_ADS = 6
        CANNOT_CREATE_TEXT_ADS = 7
        EMPTY_FIELD = 8
        RESOURCE_REFERENCED_IN_MULTIPLE_OPS = 9
        AD_TYPE_CANNOT_BE_PAUSED = 10
        AD_TYPE_CANNOT_BE_REMOVED = 11
        CANNOT_UPDATE_DEPRECATED_ADS = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import geo_target_constant_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "GeoTargetConstant",
    },
)


class GeoTargetConstant(proto.Message):
    r"""A geo target constant.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the geo target constant.
            Geo target constant resource names have the form:

            ``geoTargetConstants/{geo_target_constant_id}``
        id (int):
            Output only. The ID of the geo target
            constant.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            Output only. Geo target constant English
            name.

            This field is a member of `oneof`_ ``_name``.
        country_code (str):
            Output only. The ISO-3166-1 alpha-2 country
            code that is associated with the target.

            This field is a member of `oneof`_ ``_country_code``.
        target_type (str):
            Output only. Geo target constant target type.

            This field is a member of `oneof`_ ``_target_type``.
        status (google.ads.googleads.v20.enums.types.GeoTargetConstantStatusEnum.GeoTargetConstantStatus):
            Output only. Geo target constant status.
        canonical_name (str):
            Output only. The fully qualified English
            name, consisting of the target's name and that
            of its parent and country.

            This field is a member of `oneof`_ ``_canonical_name``.
        parent_geo_target (str):
            Output only. The resource name of the parent geo target
            constant. Geo target constant resource names have the form:

            ``geoTargetConstants/{parent_geo_target_constant_id}``

            This field is a member of `oneof`_ ``_parent_geo_target``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    target_type: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    status: (
        geo_target_constant_status.GeoTargetConstantStatusEnum.GeoTargetConstantStatus
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=geo_target_constant_status.GeoTargetConstantStatusEnum.GeoTargetConstantStatus,
    )
    canonical_name: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    parent_geo_target: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This code example uploads an image asset.

To get image assets, run get_all_image_assets.py.
"""


import argparse
import sys

from examples.utils.example_helpers import get_image_bytes_from_url
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.asset_service import AssetOperation
from google.ads.googleads.v22.resources.types.asset import Asset


# [START upload_image_asset]
def main(client: GoogleAdsClient, customer_id: str) -> None:
    """Main method, to run this code example as a standalone application."""

    # Download image from URL
    url: str = "https://gaagl.page.link/Eit5"
    image_content: bytes = get_image_bytes_from_url(url)

    asset_service = client.get_service("AssetService")
    asset_operation: AssetOperation = client.get_type("AssetOperation")
    asset: Asset = asset_operation.create
    asset.type_ = client.enums.AssetTypeEnum.IMAGE
    asset.image_asset.data = image_content
    asset.image_asset.file_size = len(image_content)
    asset.image_asset.mime_type = client.enums.MimeTypeEnum.IMAGE_JPEG
    # Use your favorite image library to determine dimensions
    asset.image_asset.full_size.height_pixels = 315
    asset.image_asset.full_size.width_pixels = 600
    asset.image_asset.full_size.url = url
    # Provide a unique friendly name to identify your asset.
    # When there is an existing image asset with the same content but a different
    # name, the new name will be dropped silently.
    asset.name = "Marketing Image"

    mutate_asset_response = asset_service.mutate_assets(
        customer_id=customer_id, operations=[asset_operation]
    )
    print("Uploaded file(s):")
    for row in mutate_asset_response.results:
        print(f"\tResource name: {row.resource_name}")

    # [END upload_image_asset]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Upload an image asset from a URL."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)
